#!/usr/bin/env python
from numpy import zeros, arange, pi, e, ceil, meshgrid
from matplotlib.pyplot import imread, imshow, plot, xlabel, ylabel, show, \
        subplot, xlim, savefig
from mpl_toolkits.mplot3d import Axes3D
from scipy.ndimage import convolve
from time import time
from sys import argv, exit

def exit_with_usage():
    """Print an error message with the program's usage and exit the program."""
    print 'Usage: python %s timer REPEAT | diff SCALE' % argv[0]
    exit(1)

def Gauss(s):
    """Create a sampled Gaussian function of scale s."""
    size = int(ceil(3 * s))
    r = 2 * size + 1
    W = zeros((r, r))
    t = s ** 2.
    a = 1 / (2 * pi * t)

    # Sample the Gaussian function
    for x in xrange(r):
        for y in xrange(r):
            W[x, y] = a * e ** -(((x - size) ** 2 + (y - size) ** 2) / (2 * t))

    # Make sure that the sum of all kernel values is equal to one
    return W / W.sum()

if len(argv) < 2:
    exit_with_usage()

F = imread('cameraman.png')

if argv[1] == 'timer':
    # Time for multiple scales
    S = [1, 2, 3, 5, 7, 9, 11, 15, 19]
    repeat = int(argv[2])
    timings = []

    for i, s in enumerate(S):
        t = 0

        for k in xrange(repeat):
            W = Gauss(s)
            start = time()
            convolve(F, W, mode='nearest')
            t += time() - start

        timings.append(t / repeat)

    xlim(S[0], S[-1])
    xlabel('s')
    ylabel('time (s)')
    plot(S, timings, 'o-')
elif argv[1] == 'diff':
    # Calculate and plot the convolution of the given scale
    if len(argv) < 3:
        exit_with_usage()

    s = float(argv[2])
    W = Gauss(s)
    G = convolve(F, W, mode='nearest')

    # Original image
    subplot(131)
    imshow(F, cmap='gray')

    # Gauss function (3D plot)
    x = arange(W.shape[0])
    X, Y = meshgrid(x, x)
    ax = subplot(132, projection='3d')
    stride = s / 4
    ax.plot_surface(X, Y, W, rstride=stride, cstride=stride, cmap='jet')
    ax.set_xlabel('x')
    ax.set_ylabel('y')
    ax.set_zlabel('g(x, y)')

    # Convolution
    subplot(133)
    imshow(G, cmap='gray')

show()
